/*
* Global variables for C plotting package
*
* device  :  Graphics device number
* termin  :  Zero for noninteractive device, non-zero for terminal
* graphx  :  Zero if currently in text mode, non-zero for graphics mode
* currx   :  Physical x-coordinate of current point
* curry   :  Physical y-coordinate of current point
* vpp...  :  Viewport boundaries in physical coordinates
* spp...  :  Subpage  boundaries in physical coordinates
* clp...  :  Clip     boundaries in physical coordinates
* phy...  :  Physical device limits in physical coordinates
* nsub.   :  Number of subpages on physical device
* cursub  :  Current subpage
* um.     :  Number of micrometres in a pixel
* font    :  Current default font
* colour  :  Current default colour
* mark    :  Array of mark lengths in micrometres for broken lines
* space   :  Array of space lengths in micrometres for broken lines
* nms     :  Number of elements for current broken line style
* timecnt :  Timer for broken lines
* alarm   :  Alarm indicating change of broken line status
* pendn   :  Flag indicating if pen is up or down
* curel   :  Current element within broken line
* level   :  Initialization level
*
* spd...  :  Subpage  boundaries in normalized device coordinates
* vpd...  :  Viewport boundaries in normalized device coordinates
* vpw...  :  Viewport boundaries in world coordinates
* .pmm    :  Number of pixels to a millimetre
* wp....  :  Transformation variables for world  to physical conversion
* dp....  :  Transformation variables for device to physical conversion
* mp....  :  Transformation variables for millimetres from bottom left
*             hand corner to physical coordinates
* wm....  :  Transformation variables for world coordinates to mm
* chr...  :  Character default height and current (scaled) height
* sym...  :  Symbol    default height and current (scaled) height
* maj...  :  Major tick default height and current (scaled) height
* min...  :  Minor tick default height and current (scaled) height
* base3.  :  World coordinate size of base for 3-d plot
* basec.  :  Position of centre of base for 3-d plot
* dom...  :  Minimum and maximum values for domain
* zzscl   :  Vertical (z) scale for 3-d plot
* ran..   :  Minimum and maximum z values for 3-d plot
* c..     :  Coordinate transformation from 3-d to 2-d
*/

#include <stdio.h>  /* Just for NULL definition */

int device, termin, graphx;
int currx,  curry;
int vppxmi, vppxma, vppymi, vppyma;
int sppxmi, sppxma, sppymi, sppyma;
int clpxmi, clpxma, clpymi, clpyma;
int phyxmi, phyxma, phyymi, phyyma;
int nsubx,  nsuby, cursub;
int umx,    umy;
int font,   colour, style;
int mark[10], space[10], nms, timecnt;
int alarm,  pendn,  curel;
int level=0;

int *heap3=NULL;
int *heapc=NULL;

float spdxmi, spdxma, spdymi, spdyma;
float vpdxmi, vpdxma, vpdymi, vpdyma;
float vpwxmi, vpwxma, vpwymi, vpwyma;
float xpmm,   ypmm;
float wpxscl, wpxoff, wpyscl, wpyoff;
float dpxscl, dpxoff, dpyscl, dpyoff;
float mpxscl, mpxoff, mpyscl, mpyoff;
float wmxscl, wmxoff, wmyscl, wmyoff;
float chrdef, chrht;
float symdef, symht;
float majdef, majht;
float mindef, minht;
float base3x, base3y, basecx, basecy;
float domxmi, domxma, domymi, domyma;
float zzscl,  ranmi,  ranma;
float cxx,    cxy,    cyx,    cyy,   cyz;
